{% load i18n %}{% autoescape off %}
{% blocktranslate %}You're receiving this email because an account ahs been created for you at {{ site_name }}.{% endblocktranslate %}

{% translate "Please go to the following page and choose a password:" %}
{% block reset_link %}
{{ reset_link }}
{% endblock %}
{% translate 'Your username:' %} {{ user.get_username }}

{% translate "Thanks for using our site!" %}

{% blocktranslate %}The {{ site_name }} team{% endblocktranslate %}

{% endautoescape %}
