{% load i18n %}{% autoescape off %}
{% blocktranslate %}You're receiving this email because an account ahs been created for you at {{ site_name }}.{% endblocktranslate %}

<h1>
{% translate "Please go to the following page and choose a password:" %}
</h1>
{% block reset_link %}
<a href="{{ reset_link }}">Set Password</a>
{% endblock %}
<br>
{% translate 'Your username:' %} {{ user.get_username }}
<br>

{% translate "Thanks for using our site!" %}
<br>

{% blocktranslate %}The {{ site_name }} team{% endblocktranslate %}

{% endautoescape %}
